#! /bin/python

import os
import time
import simulator

BATCH_SIZE = 100
START_AT = 0

SIMULATOR_PATH = "/media/jbm/JBM-WORK/CompSciMSc/Dissertation/simulators/hacksim3/target/release/hacksim3"
CONFIGS_PATH = "/home/jbm/dissertation/select_configs/"
OUTPUT = "select.csv"

HEADING = "file, rom, control_unit, a_instruction_unit, memory_unit, operand_unit, arithmetic_logic_unit, jump_unit, write_back_unit, decode_unit, time_elapsed, instructions, cycles, IPC, CPI, stalls, flushes, half_flushes, cache_hit, cache_compulsory, cache_capacity, outcome_accuracy, taken_accuracy, not_taken_accuracy, target_accuracy, branch_accuracy"

def write_to_csv(lines):
    with open(OUTPUT, "a") as output:
        for line in lines:
            output.write(line + "\n")

def prepare_csv():
    with open(OUTPUT, "w") as output:
        output.truncate(0)
        output.write(HEADING + "\n")
        
def main():
    if START_AT == 0:
        prepare_csv()

    sim_files = next(os.walk(CONFIGS_PATH), (None, None, []))[2]
    sim_files.sort()

    counter = 0
    i = START_AT
    while i < len(sim_files):
        start_time = time.time()

        lines = []

        start = i
        end = i + BATCH_SIZE

        if end > len(sim_files):
            end = len(sim_files)

        #queue up BATCH_SIZE number of threads
        threads = []
        for j in range(start, end):
            thread = simulator.Runner(SIMULATOR_PATH, CONFIGS_PATH, sim_files[j])
            threads.append(thread)

        #start the batch
        for thread in threads:
            thread.start()

        #wait for batch to end
        for thread in threads:
            thread.join()
            lines.append(thread.line)

        write_to_csv(lines)

        end_time = time.time()
        print(f'Batch {counter} (items {start} - {end - 1}) completed in {end_time - start_time} seconds')

        i += BATCH_SIZE
        counter += 1

if __name__ == "__main__":
    start = time.time()

    main()

    end = time.time()

    print(f'Time elapsed: {end - start} seconds')