CONTROL_UNITS = ["staged", "pipeline_forwarding"]
A_INSTRUCTION_UNITS = ["basic"]
MEMORY_UNITS = ["basic", "static_16", "direct_mapped_16", "n_way_set_associative_2_16_fifo", "fully_associative_16_lrecent"]
MEMORY_UNIT_SIZES = ["8", "16", "32", "64", "128", "256"]
MEMORY_UNIT_STRATEGIES = ["lifo", "fifo", "lrecent", "mrecent"]
MEMEORY_UNIT_WAYS_LINES = [("2", "8"), ("4", "8"), ("2", "16"), ("4", "16"), ("8", "16"), ("2", "32"), ("4", "32"), ("8", "32"), ("16", "32"), ("2", "64"), ("4", "64"), ("8", "64"), ("16", "64"), ("2", "128"), ("4", "128"), ("8", "128"), ("16", "128"), ("2", "256"), ("4", "256"), ("8", "256"), ("16", "256")]
OPERAND_UNITS = ["basic", "forwarding"]
ARITHMETIC_LOGIC_UNITS = ["basic"]
JUMP_UNITS = ["basic"]
WRITE_BACK_UNITS = ["basic", "simultaneous"]
DECODE_UNITS = ["basic", "local1bit16_fifo1bit32", "local2bit16_fifo1bit32", "local2bit16_fifo2bit32", "gshare16_fifo2bit32"]
OUTCOME_PREDICTORS = ["alwaystaken", "nevertaken", "global1bit", "global2bit", "local1bit[size]", "local2bit[size]", "gshare[size]"]
OUTCOME_SIZES = ["8", "16", "32", "64", "128", "256"]
TARGET_PREDICTORS = ["[strategy]1bit[size]", "[strategy]2bit[size]"]
TARGET_STRATEGIES = ["lifo", "fifo", "lrecent", "mrecent"]
TARGET_SIZES = ["8", "16", "32", "64", "128", "256"]

def expand_memory_units():
    memory_units = []

    for template in MEMORY_UNITS:
        if "_[ways]_[lines]_[strategy]" in template:
            for (ways, lines) in MEMEORY_UNIT_WAYS_LINES:
                for strategy in MEMORY_UNIT_STRATEGIES:
                    memory_units.append(template.replace("[ways]", ways).replace("[lines]", lines).replace("[strategy]", strategy))

        elif "_[size]_[strategy]" in template:
            for size in MEMORY_UNIT_SIZES:
                for strategy in MEMORY_UNIT_STRATEGIES:
                    memory_units.append(template.replace("[size]", size).replace("[strategy]", strategy))

        elif "_[size]" in template:
            for size in MEMORY_UNIT_SIZES:
                memory_units.append(template.replace("[size]", size))

        else:
            memory_units.append(template)

    return memory_units

def expand_outcome_predictors():
    outcome_predictors = []

    for template in OUTCOME_PREDICTORS:
        if "[size]" in template:
            for size in OUTCOME_SIZES:
                outcome_predictors.append(template.replace("[size]", size))

        else:
            outcome_predictors.append(template)

    return outcome_predictors

def expand_target_predictors():
    target_predictors = []

    for template in TARGET_PREDICTORS:
        for strategy in TARGET_STRATEGIES:
            for size in TARGET_SIZES:
                target_predictors.append(template.replace("[strategy]", strategy).replace("[size]", size))

    return target_predictors

def expand_decode_units():
    decode_units = []

    for template in DECODE_UNITS:
        if "[outcome]_[target]" in template:
            for outcome in expand_outcome_predictors():
                for target in expand_target_predictors():
                    decode_units.append(template.replace("[outcome]", outcome).replace("[target]", target))

        else:
            decode_units.append(template)

    return decode_units

def generate_configurations(): 
    configurations = []

    for control_unit in CONTROL_UNITS:
        for a_unit in A_INSTRUCTION_UNITS:
            for memory_unit in expand_memory_units():
                for operand_unit in OPERAND_UNITS:
                    for arithmetic_logic_unit in ARITHMETIC_LOGIC_UNITS:
                        for jump_unit in JUMP_UNITS:
                            for write_back_unit in WRITE_BACK_UNITS:
                                for decode_unit in expand_decode_units():
                                    config = ""

                                    config += (f'control_unit = "{control_unit}"\n')
                                    config += (f'a_instruction_unit = "{a_unit}"\n')
                                    config += (f'memory_unit = "{memory_unit}"\n')

                                    if operand_unit == "basic":
                                        config += (f'operand_unit = "{operand_unit}"\n')
                                    else:
                                        if control_unit == "pipeline_forwarding":
                                            config += (f'operand_unit = "{operand_unit}"\n')
                                        else:
                                            continue

                                    config += (f'arithmetic_logic_unit = "{arithmetic_logic_unit}"\n')
                                    config += (f'jump_unit = "{jump_unit}"\n')

                                    if write_back_unit == "basic":
                                        config += (f'write_back_unit = "{write_back_unit}"\n')
                                    else:
                                        if control_unit == "pipeline_forwarding":
                                            config += (f'write_back_unit = "{write_back_unit}"\n')
                                        else:
                                            continue
                                    
                                    if decode_unit == "basic":
                                        config += (f'decode_unit = "{decode_unit}"\n')
                                    else:
                                        if control_unit == "pipeline_forwarding":
                                            config += (f'decode_unit = "{decode_unit}"\n')
                                        else:
                                            continue
                                    
                                    if config != "":
                                        configurations.append(config)

    return configurations