#! /bin/python

import time
import subprocess  
import configurations
import threading

MULTITHREADED = True

TEMPLATE = "template.sim"
TEMPLATE_ROM_PATH = "[ROM_PATH]"
TEMPLATE_CONFIGURATION = "[CONFIGURATION]"

CONFIGS_PATH = "/home/jbm/dissertation/select_configs"

BIN_PATH = "/home/jbm/dissertation/bin/"
ROMS = ["boot.hack", "array_fill.hack", "function_calls.hack", "head_recursion.hack", "long_if.hack", "loop.hack", "mathematics.hack", "mem_access.hack", "mem_read.hack", "mem_write.hack", "objects.hack", "output.hack", "Pong.hack", "Seven.hack", "tail_recursion.hack"]

def generate_sim_files():
    with open(TEMPLATE, "r") as template_file:
        #load template
        template = template_file.read()

        #generate configurations
        configs = configurations.generate_configurations()

        sim_files = []

        counter = 0
        for rom in ROMS:
            for config in configs:
                rom_path = BIN_PATH + rom
                sim_file = template.replace(TEMPLATE_ROM_PATH, rom_path).replace(TEMPLATE_CONFIGURATION, config)

                sim_files.append((counter, sim_file))
                counter += 1

        return sim_files

def clear_configs():
    try:
        output = subprocess.check_output(f'rm {CONFIGS_PATH}', shell=True, stderr=subprocess.STDOUT)
    except:
        #its fine if the folder is already empty
        ...

def populate_configs_single(sim_files):
    print(f'Writing {len(sim_files)} configuration files to {CONFIGS_PATH}')

    clear_configs()

    for (id, file) in sim_files:
        write_config_file(id, file)

def populate_configs_multi(sim_files):
    print(f'Writing {len(sim_files)} configuration files to {CONFIGS_PATH}')

    clear_configs()

    for (id, file) in sim_files:
        thread = threading.Thread(target=write_config_file, args=(id, file))
        thread.start()
        

def write_config_file(id, file):
    filename = f'{CONFIGS_PATH}/{id}.sim'

    with open(filename, "w") as output:
        output.write(file)

def main():
    start = time.time()

    sim_files = generate_sim_files()

    if MULTITHREADED:
        populate_configs_multi(sim_files)
    else:
        populate_configs_single(sim_files)

    end = time.time()

    print(f'Time elapsed: {end - start} seconds')


if __name__ == "__main__":
    main()