import threading
import subprocess

class DataPoint():
    def __init__(self, config_path, sim_file):
        self.config_path = config_path
        self.sim_file = sim_file
        self.parse_sim_file()

    def parse_sim_file(self):
        with open(self.config_path + "/" + self.sim_file, "r") as file:
            lines = file.read().split("\n")
            
            rom_path = lines[1].replace("path = ", "").replace("\"", "")
            last_slash = rom_path.rfind("/")
            if last_slash == -1:
                self.rom = rom_path
            else:
                self.rom = rom_path[last_slash + 1:]

            self.control_unit = lines[11].replace("control_unit = ", "").replace("\"", "")
            self.a_instruction_unit = lines[12].replace("a_instruction_unit = ", "").replace("\"", "")
            self.memory_unit = lines[13].replace("memory_unit = ", "").replace("\"", "")
            self.operand_unit = lines[14].replace("operand_unit = ", "").replace("\"", "")
            self.arithmetic_logic_unit = lines[15].replace("arithmetic_logic_unit = ", "").replace("\"", "")
            self.jump_unit = lines[16].replace("jump_unit = ", "").replace("\"", "")
            self.write_back_unit = lines[17].replace("write_back_unit = ", "").replace("\"", "")
            self.decode_unit = lines[18].replace("decode_unit = ", "").replace("\"", "")

    def parse_output(self, output):
        lines = output.decode("utf-8").split("\n")

        self.time_elapsed = lines[19].replace("Total time elapsed: ", "").replace(" seconds", "")
        self.instructions = lines[20].replace("Total instructions: ", "")
        self.cycles = lines[22].replace("Total cycles: ", "")
        self.IPC = lines[25].replace("Average Instructions per Cycle (IPC): ", "")
        self.CPI = lines[26].replace("Average Cycles per Instruction (CPI): ", "")

        self.stalls = lines[2].replace("Stalls: ", "")
        self.flushes = lines[3].replace("Flushes: ", "")
        self.half_flushes = lines[4].replace("Half Flushes: ", "")

        self.cache_hit = lines[6].replace("Cache hit rate: ", "").replace("%", "")
        self.cache_compulsory = lines[7].replace("Cache compulsory miss rate: ", "").replace("%", "")
        self.cache_capacity = lines[8].replace("Cache capacity miss rate: ", "").replace("%", "")

        self.outcome_accuracy = lines[10].replace("Outcome prediction accuracy: ", "").replace("%", "")
        self.taken_accuracy = lines[11].replace("Outcome taken accuracy: ", "").replace("%", "")
        self.not_taken_accuracy = lines[12].replace("Outcome not taken accuracy: ", "").replace("%", "")

        self.target_accuracy = lines[14].replace("Target prediction accuracy: ", "").replace("%", "")

        self.branch_accuracy = lines[16].replace("Branch prediction accuracy: ", "").replace("%", "")


    def to_csv_line(self):
        return f'{self.sim_file}, {self.rom}, {self.control_unit}, {self.a_instruction_unit}, {self.memory_unit}, {self.operand_unit}, {self.arithmetic_logic_unit}, {self.jump_unit}, {self.write_back_unit}, {self.decode_unit}, {self.time_elapsed}, {self.instructions}, {self.cycles}, {self.IPC}, {self.CPI}, {self.stalls}, {self.flushes}, {self.half_flushes}, {self.cache_hit}, {self.cache_compulsory}, {self.cache_capacity}, {self.outcome_accuracy}, {self.taken_accuracy}, {self.not_taken_accuracy}, {self.target_accuracy}, {self.branch_accuracy}'


class Runner(threading.Thread):
    def __init__(self, sim_path, config_path, sim_file):
        threading.Thread.__init__(self)
        self.sim_path = sim_path
        self.config_path = config_path
        self.sim_file = sim_file

    def run(self):
        datapoint = DataPoint(self.config_path, self.sim_file)

        sim_output = self.simulate()
        datapoint.parse_output(sim_output)

        self.line = datapoint.to_csv_line()

    def simulate(self):
        try:
            output = subprocess.check_output([self.sim_path, self.config_path+self.sim_file], stderr=subprocess.STDOUT)
            return output
        except:
            self.line = f'{self.sim_file}, ERROR'
            print(f'ERROR simulating file {self.sim_file}')
            exit()